<?php

namespace Drupal\Tests\purge_users\Functional\policy;

use Drupal\purge_users\Entity\PurgeUsersPolicy;
use Drupal\Tests\purge_users\Functional\NotificationTest;

/**
 * Test all purge notifications.
 *
 * @group purge_users
 */
class PolicyNotificationTest extends NotificationTest {

  /**
   * A policy entity.
   *
   * @var \Drupal\purge_users\Entity\PurgeUsersPolicy
   */
  protected $policy;

  /**
   * {@inheritdoc}
   */
  public function setUpConfig(): void {
    $this->confirmFormUrl = '/admin/config/people/purge-users/policies/confirm/test_policy';
    // Create a new PurgeUsersPolicy entity instance.
    /** @var \Drupal\purge_users\Entity\PurgeUsersPolicy $policy */
    $this->policy = PurgeUsersPolicy::create([
      'id' => 'test_policy',
      'label' => 'Test Policy',
      'description' => 'A policy for testing purposes.',
      'purge_user_cancel_method' => 'user_cancel_block',
      'disregard_blocked_users' => FALSE,
      'purge_on_cron' => TRUE,
      // The text must be like this, to match the text in the assertions.
      'send_email_notification' => 'enabled',
      'inactive_user_notify_subject' => 'Dear user',
      'inactive_user_notify_text' => 'Dear User, Your account has been deleted due the website’s policy to automatically remove users who match certain criteria. If you have concerns regarding the deletion, please talk to the administrator of the website. Thank you',
      // Send pre-notification after 2 days.
      'send_email_user_before_notification' => 'enabled',
      'user_before_notification_value' => '2',
      'user_before_notification_period' => 'days',
      // Set pre-notify email subject + body.
      'user_before_deletion_subject' => 'Dear user',
      'user_before_deletion_text' => 'Dear User, Your account will be deleted soon due the website’s policy to automatically remove users who match certain criteria. If you have concerns regarding the deletion, please talk to the administrator of the website. Thank you',
    ]);
    $this->policy->save();
  }

  /**
   * {@inheritdoc}
   */
  protected function setUpConfigNoEmails() {
    $this->policy
      ->set('send_email_user_before_notification', 'disabled')
      ->set('send_email_notification', 'disabled')
      ->save();
  }

  /**
   * Sends pre-notifications to all test users.
   */
  protected function preNotifyUsers(): void {
    $this->setUpNotLoggedInUsersPolicy();
    $this->notifyManager->notifyUser($this->lastloginUser7Days, $this->policy);
    $this->notifyManager->notifyUser($this->lastloginUser4Days, $this->policy);
    $this->setUpNeverLoggedInUsersPolicy();
    $this->notifyManager->notifyUser($this->neverLoggedUser7Days, $this->policy);
    $this->notifyManager->notifyUser($this->neverLoggedUser4Days, $this->policy);
    $this->setUpBlockedUsersPolicy();
    $this->notifyManager->notifyUser($this->blockedUser7Days, $this->policy);
    $this->notifyManager->notifyUser($this->blockedUser4Days, $this->policy);
    $this->setUpInactiveUsersPolicy();
    $this->notifyManager->notifyUser($this->inactiveUser7Days, $this->policy);
    $this->notifyManager->notifyUser($this->inactiveUser4Days, $this->policy);
  }

  /**
   * Purges users through the admin UI or cron.
   *
   * @param string $mode
   *   One of 'form' or 'cron'.
   */
  protected function runPurgeAllOperation(string $mode): void {
    $this->purgeNotLoggedInUsers($mode);
    $this->purgeNeverLoggedInUsers($mode);
    $this->purgeBlockedUsers($mode);
    $this->purgeInactiveUsers($mode);

  }

  /**
   * Tests that a user who must be purged only receives the deletion email.
   */
  protected function setUpUserDeletionNotificationConfig() {
    $conditions = $this->getDefaultConditions();
    // Purge inactive users after 10 days.
    $conditions['purge_users:inactive'] = [
      'id' => 'purge_users:inactive',
      'user_inactive_value' => '10',
      'user_inactive_period' => 'days',
    ];
    $this->policy->set('policy_conditions', $conditions);

    $this->policy
      // Send pre-notification after 8 days.
      ->set('user_before_notification_value', '8')
      ->set('user_before_notification_period', 'days')
      ->set('send_email_user_before_notification', 'enabled')
      // Send purge email.
      ->set('send_email_notification', 'enabled')
      // Change the purge method, to be sure the user is being deleted.
      ->set('purge_user_cancel_method', 'user_cancel_delete');

    $this->policy->save();
  }

  /**
   * {@inheritdoc}
   */
  protected function setUpUserAccountSettingsNotificationConfig() {
    $this->config('user.settings')
      // Send core user module notifications when user is blocked.
      ->set('notify.status_blocked', TRUE)
      ->save();

    $this->policy
      ->set('send_email_notification', 'disabled')
      ->save();
  }

  /**
   * Set up the policy to purge not logged in users.
   */
  protected function setUpNotLoggedInUsersPolicy() {
    $conditions = $this->getDefaultConditions();
    // Purge regular users after 5 days of not logging in.
    $conditions['purge_users:not_logged_in'] = [
      'id' => 'purge_users:not_logged_in',
      'user_lastlogin_value' => '5',
      'user_lastlogin_period' => 'days',
    ];
    $this->policy->set('policy_conditions', $conditions);
    $this->policy->save();
  }

  /**
   * Purge users who are not logged in.
   *
   * @param string $mode
   *   One of 'form' or 'cron'.
   */
  protected function purgeNotLoggedInUsers(string $mode) {
    $this->setUpNotLoggedInUsersPolicy();
    $this->runPurgeOperation($mode);
  }

  /**
   * Set up the policy to purge never-logged-in users.
   */
  protected function setUpNeverLoggedInUsersPolicy() {
    $conditions = $this->getDefaultConditions();
    // Purge never-logged-in users after 5 days.
    $conditions['purge_users:never_logged_in'] = [
      'id' => 'purge_users:never_logged_in',
      'user_never_lastlogin_value' => '5',
      'user_never_lastlogin_period' => 'days',
    ];
    $this->policy->set('policy_conditions', $conditions);
    $this->policy->save();
  }

  /**
   * Purge users who have never logged in.
   *
   * @param string $mode
   *   One of 'form' or 'cron'.
   */
  protected function purgeNeverLoggedInUsers(string $mode) {
    $this->setUpNeverLoggedInUsersPolicy();
    $this->runPurgeOperation($mode);
  }

  /**
   * Set up the policy to purge blocked users.
   */
  protected function setUpBlockedUsersPolicy() {
    $conditions = $this->getDefaultConditions();
    // Purge blocked users after 5 days.
    $conditions['purge_users:blocked'] = [
      'id' => 'purge_users:blocked',
      'user_blocked_value' => '5',
      'user_blocked_period' => 'days',
    ];
    $this->policy->set('policy_conditions', $conditions);
    $this->policy->save();
  }

  /**
   * Purge users who are blocked.
   *
   * @param string $mode
   *   One of 'form' or 'cron'.
   */
  protected function purgeBlockedUsers(string $mode) {
    $this->setUpBlockedUsersPolicy();
    $this->runPurgeOperation($mode);
  }

  /**
   * Set up the policy to purge inactive users.
   */
  protected function setUpInactiveUsersPolicy() {
    $conditions = $this->getDefaultConditions();
    // Purge inactive users after 5 days.
    $conditions['purge_users:inactive'] = [
      'id' => 'purge_users:inactive',
      'user_inactive_value' => '5',
      'user_inactive_period' => 'days',
    ];
    $this->policy->set('policy_conditions', $conditions);
    $this->policy->save();
  }

  /**
   * Purge users who are inactive.
   *
   * @param string $mode
   *   One of 'form' or 'cron'.
   */
  protected function purgeInactiveUsers(string $mode) {
    $this->setUpInactiveUsersPolicy();
    $this->runPurgeOperation($mode);
  }

  /**
   * Get the default conditions for the policy.
   */
  private function getDefaultConditions() {
    return [
      'purge_users:excluded_roles' => [
        'id' => 'purge_users:excluded_roles',
        'purge_excluded_users_roles' => ['administrator'],
      ],
      'purge_users:included_roles' => [
        'id' => 'purge_users:included_roles',
        'purge_included_users_roles' => ['authenticated'],
      ],
    ];
  }

}
